@@ -1,5 +1,5 @@
 ;;;; File: g-wrap.scm
-;;;; Copyright (C) 2004-2006 Andreas Rottmann
+;;;; Copyright (C) 2004-2007 Andreas Rottmann
 ;;;;
 ;;;; based upon G-Wrap 1.3.4,
 ;;;;   Copyright (C) 1996, 1997,1998 Christopher Lee
@@ -169,7 +169,19 @@
 ;;; Types
 ;;;
 
+(define-class <gw-type-class> (<class>))
+
+(define-method (initialize (class <gw-type-class>) initargs)
+  (next-method)
+  ;; Inherit the allowed options
+  (let-keywords initargs #t ((allowed-options '()))
+    (class-slot-set-supers-union!
+     class 'class-allowed-options allowed-options)))
+
+
 (define-class <gw-type> (<gw-item>)
+  (class-allowed-options #:init-value '() #:allocation #:each-subclass)
+  (allowed-options #:init-value '() #:init-keyword #:allowed-options)
   (name #:getter name #:init-keyword #:name)
   (class-name #:accessor class-name
               #:init-keyword #:class-name
@@ -179,7 +191,15 @@
                      #:init-value #t)
   (arguments-visible? #:getter arguments-visible?
                       #:init-keyword #:arguments-visible?
-                      #:init-value #t))
+                      #:init-value #t)
+  #:metaclass <gw-type-class>)
+
+(define-method (initialize (type <gw-type>) initargs)
+  (next-method)
+  (slot-set! type 'allowed-options
+             (lset-union eq?
+                         (slot-ref type 'class-allowed-options)
+                         (slot-ref type 'allowed-options))))
 
 (define-method (write (type <gw-type>) port)
   (let ((class (class-of type)))
@@ -335,7 +355,9 @@
   (number #:getter number #:init-keyword #:number))
 
 (define-method (visible? (self <gw-param>))
-  (>= (number self) 0))
+  (or (>= (number self) 0)
+      (and (output-param? self)
+           (arguments-visible? (type self)))))
 
 (define-method (output-param? (self <gw-param>))
   (memq 'out (options (typespec self))))
@@ -424,24 +446,20 @@
 (define-method (input-arguments (func <gw-function>))
   (filter (lambda (arg)
            (and (visible? arg)
-                (not (memq 'out (options (typespec arg))))))
+                (not (output-argument? arg))))
          (slot-ref func 'arguments)))
 
 (define-method (input-argument-count (func <gw-function>))
   (count (lambda (arg)
            (and (visible? arg)
-                (not (memq 'out (options (typespec arg))))))
+                (not (output-argument? arg))))
          (slot-ref func 'arguments)))
 
 (define-method (output-argument-count (func <gw-function>))
-  (count (lambda (arg)
-           (memq 'out (options (typespec arg))))
-         (slot-ref func 'arguments)))
+  (count output-argument? (slot-ref func 'arguments)))
 
 (define-method (output-arguments (func <gw-function>))
-  (filter (lambda (arg)
-            (memq 'out (options (typespec arg))))
-         (slot-ref func 'arguments)))
+  (filter output-argument? (slot-ref func 'arguments)))
 
 (define-method (optional-arguments (func <gw-function>))
   (let loop ((args (reverse (slot-ref func 'arguments))) (opt-args '()))
@@ -493,7 +511,7 @@
   (type (typespec arg)))
 
 (define-method (visible? (arg <gw-argument>))
-  (and (arguments-visible? (type arg)) (not (output-argument? arg))))
+  (arguments-visible? (type arg)))
 
 (define-method (output-argument? (arg <gw-argument>))
   (memq 'out (options (typespec arg))))
