@@ -1,5 +1,5 @@
 ;;;; File: guile.scm
-;;;; Copyright (C) 2004-2006 Andreas Rottmann
+;;;; Copyright (C) 2004-2007 Andreas Rottmann
 ;;;;
 ;;;; based upon G-Wrap 1.3.4,
 ;;;;   Copyright (C) 1996, 1997,1998 Christopher Lee
@@ -247,7 +247,7 @@
 			"&gw__scm_extras[" (number->string
 					    (- number
 					       *max-fixed-params*)) "]"))))
-	     ((memq 'out (options (typespec arg)))
+	     ((output-argument? arg)
 	      `(#:wrapped-var
 		,(string-append "& "(out-param-name out-number))))
 	     (else '()))))
@@ -288,7 +288,9 @@
 		   #:typespec return-typespec
 		   #:wrapped-var "&gw__scm_result"
 		   #:var "gw__result"))
-	 (scm-params (filter visible? params))
+	 (scm-params (filter (lambda (param) (and (visible? param)
+                                                  (not (output-param? param))))
+                             params))
 
 	 (function-has-wcp-args?
 	  (or (any (lambda (p) (is-a? (type p) <gw-wct>)) params)
@@ -321,13 +323,13 @@
      (if (zero? nargs)
 	 '()
 	 (list
-	  "  GWTypeSpec *typespec = NULL;\n"
-	  "  GWTypeSpec typespecs[] = { " (string-join
-					   (map
-					    (lambda (param)
-					      (typespec-cg (type param) (typespec param)))
-					    scm-params)
-					   ", ") " };\n"))
+	  "  const GWTypeSpec *typespec = NULL;\n"
+	  "  static const GWTypeSpec typespecs[] = { "
+          (string-join
+           (map (lambda (param)
+                  (typespec-cg (type param) (typespec param)))
+                scm-params)
+           ", ") " };\n"))
 
      (if (needs-result-var? return-type)
          (let ((c-value (default-c-value-for-type return-type)))
@@ -343,17 +345,19 @@
 	 (list "  SCM gw__scm_extras[" (- nargs *max-fixed-params*) "];\n")
 	 '())
 
-     (map
-      (lambda (number)
-	(list "  SCM " (out-param-name number) " = SCM_UNSPECIFIED;\n"))
-      (iota (length out-params)))
+     (filter-map
+      (lambda (number out-param)
+        (and (visible? out-param)
+             (list "  SCM " (out-param-name number) " = SCM_UNSPECIFIED;\n")))
+      (iota (length out-params))
+      out-params)
 
      "\n"
 
       (map
        (lambda (param)
 	 (list (c-type-name (type param)) " " (var param)
-               (if (memq 'out (options (typespec param)))
+               (if (output-param? param)
                    ;; Initialize the C variable for the `out' parameter if
                    ;; possible.
                    (let ((c-value
@@ -367,7 +371,7 @@
       (map
        (lambda (param arg)
 	 (list
-	  (if (visible? param)
+	  (if (and (not (output-param? param)) (visible? param))
 	      (list
 	       "/* ARG " (number param) " */\n"
 	       (if (memq 'aggregated (options (typespec param)))
@@ -440,9 +444,7 @@
 			   (if (zero? nargs)
 			       "wrapper_exit"
 			       (format #f "post_call_arg_~A" (- nargs 1))))
-	     "SCM_DEFER_INTS;\n"
-	     (expand-special-forms call-code #f '(memory misc type range))
-	     "SCM_ALLOW_INTS;\n")
+	     (expand-special-forms call-code #f '(memory misc type range)))
 	    "/* no function call requested! */\n"))
 
       "{\n/* post-call-result-cg */\n"
@@ -455,7 +457,7 @@
       ;; insert the post-call args code in the opposite order
       ;; of the pre-call code
       (map
-       (lambda (param arg)
+       (lambda (param)
 	 (list
 	  (label-cg labels (format #f "post_call_arg_~A" (number param)))
 	  (let ((post-call-code
@@ -466,7 +468,7 @@
 		'()
 		(list "{\n" post-call-code "}\n")))
 	  "}\n"))
-       (reverse params) (arguments function))
+       (reverse params))
 
       (if (and function-has-wcp-args? function-has-aggregated-args?)
 	  ;; Since some of the input arguments are aggregated by the output
@@ -487,17 +489,23 @@
       "    gw_handle_wrapper_error(NULL, &gw__error,\n"
       "                             " fn-c-string ",\n"
       "                             gw__arg_pos);\n"
-      (if (null? out-params)
-	  "  return gw__scm_result;\n"
-	  (list
-	   "  return scm_values (scm_list_n ("
-	   (if (needs-result-var? return-type)
-	       "gw__scm_result, "
-	       '())
-	   (map (lambda (n)
-		  (string-append (out-param-name n) ", "))
-		(iota (length out-params)))
-	   "SCM_UNDEFINED));\n"))
+      (cond ((null? out-params)
+             "  return gw__scm_result;\n")
+            ((and (not (needs-result-var? return-type))
+                  (null? (cdr out-params)))
+             (list "  return (" (out-param-name 0) ");\n"))
+            (else
+             (list
+              "  return scm_values (scm_list_n ("
+              (if (needs-result-var? return-type)
+                  "gw__scm_result, "
+                  '())
+              (filter-map (lambda (n out-param)
+                            (and (visible? out-param)
+                                 (string-append (out-param-name n) ", ")))
+                          (iota (length out-params))
+                          out-params)
+              "SCM_UNDEFINED));\n")))
       "}\n")))
 
 ;; RTI functions override this method
@@ -505,7 +513,7 @@
 				   (function <gw-guile-function>)
 				   status-var)
 
-  (let* ((visible-args (filter visible? (arguments function)))
+  (let* ((visible-args (input-arguments function))
 	 (n-visible-args (length visible-args))
 	 (n-optional-visible-args (length (filter identity
 						  (map default-value visible-args))))
@@ -744,8 +752,7 @@
 ;;;
 
 (define-class <gw-guile-wct> (<gw-wct> <gw-guile-rti-type>)
-  wct-var-name
-  #:allowed-options '(null-ok))
+  wct-var-name)
 
 (define-method (wrap-as-wct! (wrapset <gw-guile-wrapset>) . args)
   (let ((type (apply make <gw-guile-wct> args)))
@@ -865,7 +872,6 @@
       ";; Generated by G-Wrap-TNG: an experimental Guile C API-wrapper engine.\n"
       "\n"
       (format #f "(define-module ~S\n" guile-module)
-      (format #f "  #:use-module (oop goops)\n")
       (format #f "  #:use-module (g-wrap config)\n")
       ")\n"
       "\n"
@@ -882,7 +888,8 @@
 ;;      "(module-use! (module-public-interface (current-module)) (current-module))\n"
       )
      port)
-    (let ((gf-hash (make-hash-table 67)))
+    (let ((gf-hash       (make-hash-table 67))
+          (has-generics? #f))
       (fold-functions
        (lambda (func rest)
 	 (let ((gf-name (generic-name func)))
@@ -892,8 +899,17 @@
 		    (not (uses-rti-for-function? wrapset func)))
 	       (let ((handle
 		      (hashq-create-handle! gf-hash gf-name '())))
+                 (set! has-generics? #t)
 		 (set-cdr! handle (cons func (cdr handle)))))))
        #f wrapset)
+
+      (if has-generics?
+          (begin
+            ;; The generated module requires GOOPS.
+            (newline port)
+            (write '(use-modules (oop goops)) port)
+            (newline port)))
+
       (hash-fold
        (lambda (gf funcs rest)
          (for-each
